'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

const http = require('http');
const https = require('https');
const zlib = require('zlib');
const url = require('url');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

const http__default = /*#__PURE__*/_interopDefaultLegacy(http);
const https__default = /*#__PURE__*/_interopDefaultLegacy(https);
const zlib__default = /*#__PURE__*/_interopDefaultLegacy(zlib);

const supportedCompressions = ["gzip", "deflate"];
const adapterForHttp = (protocol) => {
  if (protocol === "http:") {
    return http__default['default'];
  }
  if (protocol === "https:") {
    return https__default['default'];
  }
  throw new Error(`Wrong url protocol: ${protocol}`);
};
class Request {
  constructor(url$1, method) {
    this.redirect = () => {
      this.redirectEnabled = true;
    };
    this.timeout = (timeout) => {
      this.timeoutTime = timeout;
    };
    this.compress = () => {
      this.compressionEnabled = true;
      if (!this.reqHeaders.hasOwnProperty("accept-encoding")) {
        this.reqHeaders["accept-encoding"] = supportedCompressions.join(", ");
      }
    };
    this.setHeader = (headerData) => {
      Object.entries(headerData).forEach(([headerKey, headerValue]) => {
        this.reqHeaders[headerKey.toLowerCase()] = String(headerValue);
      });
    };
    this.setQuery = (queryData) => {
      Object.entries(queryData).forEach(([queryKey, queryValue]) => {
        this.url.searchParams.append(queryKey, String(queryValue));
      });
    };
    this.setBody = (bodyData) => {
      this.body = JSON.stringify(bodyData);
    };
    this.send = () => new Promise((resolve, reject) => {
      if (this.body) {
        if (!this.reqHeaders.hasOwnProperty("content-type")) {
          this.reqHeaders["content-type"] = "application/json";
        }
        if (!this.reqHeaders.hasOwnProperty("content-length")) {
          this.reqHeaders["content-length"] = String(Buffer.byteLength(this.body));
        }
      }
      const options = {
        protocol: this.url.protocol,
        host: this.url.hostname,
        port: this.url.port,
        path: `${this.url.pathname}${this.url.search === null ? "" : this.url.search}`,
        method: this.method.toUpperCase(),
        headers: this.reqHeaders
      };
      const req = adapterForHttp(options.protocol).request(options, (res) => {
        if (res.headers.location && this.redirectEnabled) {
          this.url = new url.URL(res.headers.location, this.url.toString());
          this.send().then(resolve).catch(reject);
        } else {
          let stream = res;
          if (this.compressionEnabled) {
            const encodingType = res.headers["content-encoding"];
            if (encodingType === "gzip") {
              stream = res.pipe(zlib__default['default'].createGunzip());
            } else if (encodingType === "deflate") {
              stream = res.pipe(zlib__default['default'].createInflate());
            }
          }
          resolve({stream, response: res});
        }
      });
      if (this.body) {
        req.write(this.body);
      }
      if (this.timeoutTime) {
        req.setTimeout(this.timeoutTime, () => {
          req.destroy();
          reject(new Error("Timeout"));
        });
      }
      req.on("error", reject);
      req.end();
    });
    this.url = new url.URL(url$1);
    this.method = method;
    this.compressionEnabled = false;
    this.redirectEnabled = false;
    this.timeoutTime = 0;
    this.reqHeaders = {};
    this.body = "";
  }
}

class Response {
  constructor(response) {
    this.addChunk = (chunk) => {
      this.body = Buffer.concat([this.body, chunk]);
    };
    this.body = Buffer.alloc(0);
    this.statusCode = response.statusCode;
  }
}
const maxBuffer = 50 * 1e6;
const defaultOptions$1 = {
  timeout: 5e3,
  followRedirects: true,
  compress: true
};
const request = (options) => new Promise((resolve, reject) => {
  const {url, method = "get", compress, followRedirects, timeout, headers, query, body} = {
    ...defaultOptions$1,
    ...options
  };
  const req = new Request(url, method);
  if (timeout)
    req.timeout(timeout);
  if (followRedirects)
    req.redirect();
  if (compress)
    req.compress();
  if (headers)
    req.setHeader(headers);
  if (query)
    req.setQuery(query);
  if (body)
    req.setBody(body);
  req.send().then(({stream, response}) => {
    const res = new Response(response);
    stream.on("error", (err) => {
      reject(err);
    });
    stream.on("data", (chunk) => {
      res.addChunk(chunk);
      if (res.body.length > maxBuffer) {
        stream.destroy();
        reject(new Error("Exceed the acceptable buffer length"));
      }
    });
    stream.on("end", () => {
      const {statusCode} = res;
      if (statusCode && statusCode >= 200 && statusCode < 300) {
        resolve(JSON.parse(res.body.toString()));
      } else {
        reject(new Error(`Server responded with ${response.statusCode}: ${response.statusMessage}`));
      }
    });
  }).catch(reject);
});

const defaultOptions = {
  timeout: 15e3,
  followRedirects: true,
  compress: true
};
const streamRequest = (options) => new Promise((resolve, reject) => {
  const {url, method = "get", compress, followRedirects, timeout, headers, query, body} = {
    ...defaultOptions,
    ...options
  };
  const req = new Request(url, method);
  if (timeout)
    req.timeout(timeout);
  if (followRedirects)
    req.redirect();
  if (compress)
    req.compress();
  if (headers)
    req.setHeader(headers);
  if (query)
    req.setQuery(query);
  if (body)
    req.setBody(body);
  req.send().then(({stream, response}) => {
    const {statusCode} = response;
    if (statusCode && statusCode >= 200 && statusCode < 300) {
      resolve(stream);
    } else {
      reject(new Error(`Server responded with ${response.statusCode}: ${response.statusMessage}`));
    }
  }).catch(reject);
});

exports.request = request;
exports.streamRequest = streamRequest;
